/**
 * \file InfoBand.hpp
 *
 * Header file that declares information band specific functions 
 *
 * \author Redouane Boumghar, Hung Cao
 *
 * \ingroup lgl
 *
 **/


#ifndef LGL_INFOBAND_HPP
#define LGL_INFOBAND_HPP

#include <iostream>

#include <gdal_priv.h>

#include <lgl/Raster.hpp>
#include <lgl/RasterBand.hpp>

namespace jafar {
  namespace lgl {
		
		/**
		 * Traversability raster band, data type in double [0,1.0] 
		 * expressing the traversability probability of each raster cell
		 *
		 * 0   : Certainty for No Obstacle 
		 * 1.0 : Certainty for Obstacle presence
		 */
    class GDInfoBand : public GDRasterBand<double> { /*{{{*/ 
			
		public:
						
			GDInfoBand(GDALDataset* _dataset, GDALRasterBand *_band);
			
			~GDInfoBand();

			// abstract methods from RasterBand
			bool isHomogeneous(const RasterRect& rect) const;			

		private:

		protected:
			
		};
		
  }
}

#endif

