/**
 * \file VisibleArea.hpp
 *
 * Header file to declare classes for the visibility computation
 * 
 *
 * \author Cyril Robin
 *
 * \ingroup lgl
 *
 **/

#ifndef LGL_VISIBLEAREA_HPP
#define LGL_VISIBLEAREA_HPP

#include <lgl/GeoData.hpp>
#include <lgl/NavGNode.hpp>

#include <set>

#ifndef DEFAULT_VIEW_LENGTH
#define DEFAULT_VIEW_LENGTH 10
#endif

// Type for the integer used as ID for the cells
#ifndef CELL_ID
#define CELL_ID

// 0 < Cell_ID < 4 294 967 295 
typedef unsigned long int Cell_ID ;

// /!\ BEWARE that if y>= ysize, there will be a problem...

#endif

namespace jafar { //namespace jafar
	namespace lgl { //namespace lgl

		/** Definition of VisibleAreaClass
		 *
		 * Contain the visible aera (set of organized vertices) of one specific vertex originalVertex).
		 * The vertices use the "NavGNode" structure.
		 */
		class VisibleArea {
		///{{{
			private :
			
				Cell_ID originalVertex ; 				// "Center" of the Area :
														// visibility is compute according to this vertex.
			
				// size of environment
				unsigned long int xsize ; // 2N+1 : vertices
				unsigned long int ysize ; // 2N+1 : vertices


			public:
					
				std::set<Cell_ID> visibleArea ; 	// Sorted list of the Vertices 
													// from which the original vertex is visible.
															
				/** Constructor
				 */
				VisibleArea() : originalVertex (0), xsize(0), ysize(0) {}

				/** Constructor
				 */
				VisibleArea(	Cell_ID _originalVertex,
								const unsigned long int &_xsize,
								const unsigned long int &_ysize
							):
								originalVertex (_originalVertex),
								xsize(_xsize),
								ysize(_ysize)
				{
					visibleArea.insert(originalVertex);
				}

				/** Destructor
				 */
				~VisibleArea(){}

				/** Update the visible area according to new data provide by the VisibleArea as argument, for the same original vertex 
				 */
				void updateArea ( const	std::set<Cell_ID> &otherArea );

				
				/** Get functions
				 */
				Cell_ID getOriginalVertex() const;

				unsigned long int getxsize() const;
				unsigned long int getysize() const;

				/** Print the content of visibleArea
				 */
				void printVisibleArea() ;

		}; ///}}} VisibleArea

		std::ostream& operator<<(std::ostream& out, const VisibleArea &_visibleArea);
	
	} // end of namespace lgl
} // end of namespace jafar

#endif /* LGL_VISIBLEAREA_HPP */
