/**
 * \file lglException.cpp
 *
 * Header file that declares LglException functions
 *
 * \author Redouane Boumghar, Hung Cao
 *
 * \ingroup lgl
 *
 **/

#ifndef LGL_LGL_EXCEPTION_HPP
#define LGL_LGL_EXCEPTION_HPP

#include "kernel/jafarException.hpp"

namespace jafar {

  namespace lgl {

    /** Base class for all exceptions defined in the module
     * lgl.
     *
     * @ingroup lgl
     */
    class LglException : public ::jafar::kernel::Exception {

    public:

      /** This enumeration defines exceptions id for the module
       * lgl.
       */
      enum ExceptionId {
        //        MY_ERROR /**< my error */
      };

      /** Constructor. You should not use this constructor directly,
       * prefer macros jfrThrowEx or jfrCreateEx which fill for you
       * parameters \c file_ and \c line_.
       *
       * @param id_ exception id
       * @param message_ message used for debug
       * @param file_ where the exception was thrown
       * @param line_ where the exception was thrown
       */
      LglException(ExceptionId id_, 
                            const std::string& message_, 
                            const std::string& file_, int line_) throw();

      virtual ~LglException() throw();

      ExceptionId getExceptionId() const throw(); 

    protected:

      ExceptionId id;

      static std::string exceptionIdToString(ExceptionId id_) throw();
      
    }; // class LglException

  } // namespace lgl
} // namespace jafar

#endif // LGL_LGL_EXCEPTION_HPP

