/**
 * \file InfoBand.cpp
 *
 * Source file that defines geodata information band specific fuctions
 *
 * \author Redouane Boumghar
 *
 * \ingroup lgl
 *
 **/

#include <lgl/InfoBand.hpp>

namespace jafar {
	namespace lgl {

		// GDInfoBand constructor
		GDInfoBand::GDInfoBand(GDALDataset* _dataset, GDALRasterBand *_band):
		GDRasterBand<double>(_dataset,_band) 
		{ }

		// GDInfoBand destructor
		GDInfoBand::~GDInfoBand() { }

		bool GDInfoBand::isHomogeneous(const RasterRect& rect) const 
		{ //{{{

			if ( (    getxsize_m(rect) > decompositionLimit
				   || getysize_m(rect) > decompositionLimit)
				&& decompositionLimit != -1.0)
			{
				// To get large cells decomposed we return non-homogeneity
				return false;
			}

			bool result = true;

			switch(homogType)
			{
				case SIGMA:
					result = sigmaSameValue(rect);
					break;
				case NEARLY:
					result = nearlySameValue(rect);
					break;
				case SIMPLE:
					// Simple equality
					result = sameValue(rect);
					break;
				default:
					return false;
			}

			return result;
		} //}}}

	}
}
