/**
 * \file Raster.cpp
 *
 * Source file for raster definitions. 
 *
 * \author Redouane Boumghar, Hung Cao
 *
 * \ingroup lgl
 *
 **/

#include <lgl/Raster.hpp>

using namespace jafar::lgl;


void Raster::setClusteringF(CLUSTERING_FUNCTION cf) {
  clusteringF=cf;
}

Raster::CLUSTERING_FUNCTION Raster::getClusteringF() {
  return clusteringF;
}

void Raster::setClusteringThres(double _cfthreshold) {
  cfthreshold=_cfthreshold;
}

double Raster::getClusteringThres() {
  return cfthreshold;
}

bool Raster::inside(const RasterRect& rect)
{ //{{{
	int _xroot = xroot;
	int _yroot = yroot;
	int _xend = _xroot+xsize;
	int _yend = _yroot-ysize;
  
	if (   rect.isValid()
	    && (rect.xtopleft >= _xroot)
	    && (rect.ytopleft >= _yroot)
	    //&& (rect.ytopleft <= _yroot)
	    && (rect.xbottomright <= _xend)
	    && (rect.ybottomright <= _yend) ) {
	    //&& (rect.ybottomright >= _yend) ) {
		return true;
	} else {
		return false;
	}
} //}}}


bool Raster::validIndex(const RasterCellIndex& index) const {
  if ( (index.i<xroot)
		 ||(index.i>=xroot+xsize)
		 ||(index.j<yroot)
		 ||(index.j>=yroot+ysize))
	 return false;

  return true;
}

RasterRect Raster::getRasterRect() const {
  return RasterRect(xroot,yroot,xsize,ysize);
}

std::ostream& jafar::lgl::operator<< (std::ostream& out, const RasterCellIndex& cell) {
  out << "(i=" << cell.i <<",j=" << cell.j << ")";
  return out;
}
