/**
 * \file VisibleArea.cpp
 *
 * Source code for VisibleArea Class 
 *
 * \author Cyril Robin 
 *
 * \ingroup lgl
 *
 **/

#include <lgl/GeoData.hpp>
#include <lgl/NavGNode.hpp>
#include <lgl/VisibleArea.hpp>

#include <ostream>
#include <set>

namespace jafar {
	namespace lgl {

//------------------------------------------------------------------------//
//-------------------- Visible Area Implementation -----------------------//
//{{{ --------------------------------------------------------------------//

		//void VisibleArea::updateArea (const std::set<NavGNode, std::less<NavGNode> > &otherArea) 
		void VisibleArea::updateArea (const std::set<Cell_ID> &otherArea) 
		{///{{{

			visibleArea.insert( otherArea.begin(), otherArea.end() );

		}///}}}

		void VisibleArea::printVisibleArea()
		{///{{{

			std::cout << " Visible Area for " << originalVertex << " is made of "<< visibleArea.size() << " vertices : " << std::endl;

			std::set<Cell_ID>::iterator it ;

			for ( it=visibleArea.begin(); it != visibleArea.end() ; ++it )
				std::cout << "(" << (*it) << ") ";

			std::cout << std::endl;
			
		}///}}}

		Cell_ID VisibleArea::getOriginalVertex() const
		{///{{{
			return ( originalVertex );
		}///}}}

		unsigned long int VisibleArea::getxsize() const
		{//{{{
			return( xsize );
		}//}}}

		unsigned long int VisibleArea::getysize() const
		{//{{{
			return( ysize );
		}//}}}

		//void Display();

//}}} ------------------ /Visible Area Implementation --------------------//

//------------------------------------------------------------------------//
//--------------------------- OUTPUT STREAM  -----------------------------//
//{{{ --------------------------------------------------------------------//

		std::ostream& operator<<(std::ostream& out, const VisibleArea &rhs)
		{///{{{
			//std::set<NavGNode>::const_iterator	
			std::set<Cell_ID>::const_iterator	
				sit ( rhs.visibleArea.begin() ),
				send( rhs.visibleArea.end()   );

			out << "---III--- Node #"  << rhs.getOriginalVertex() << " is visible from :" << std::endl ; 
			for(;sit!=send;++sit) 
				out << "---III--- ******* " <<  *sit << std::endl;
  			out << std::endl;

			return (out) ;
		}///}}}
		

//}}} ---------------------- /OUTPUT STREAM  -----------------------------//

	}
}
